#ifndef CHESSY_MOVEGENERATION_H
#define CHESSY_MOVEGENERATION_H

#include "BitBoard.h"
#include <string>
#include <array>

namespace chessy
{
    struct Move;

    // forward declaration
    class ChessGame;

    class PositionSet;

    const int WHITE_SIDE = 0;
    const int BLACK_SIDE = 1;

    template<int side>
    class PawnPushGenerator;

    class KnightMoveGenerator;
    class RookMoveGenerator;
}


struct chessy::Move
{
    Index origin;
    Index destination;

    Move(void)          = default;
    Move(const Move&)   = default;
    ~Move(void)         = default;

    inline Move(Index origin, Index destination) :
        origin{origin}, destination{destination} {}

    std::string asString(void) const;
};


class chessy::PositionSet
{
    Bitboard bitboard;

public:

    inline PositionSet(Bitboard b) : bitboard{ b } {}

    struct PositionSetIterator
    {
        Bitboard bitboard;

        inline Index operator *(void) const
        {
            return Index{ int8_t(trailingZeroes(bitboard.bits)) };
        }

        inline void operator ++(void)
        {            
            bitboard.bits &= bitboard.bits - 1ULL; // remove least significant one bit
        }

        inline bool operator !=(const PositionSetIterator& psi) const
        {
            return bitboard != psi.bitboard;
        }
    };

    inline PositionSetIterator begin(void) const
    {
        return PositionSetIterator{ bitboard };
    }

    inline static PositionSetIterator end(void)
    {
        return PositionSetIterator{ 0 };
    }
};


template<int side>
class chessy::PawnPushGenerator
{
    const ChessGame& chessGame;
    struct MoveIterator
    {
        PositionSet::PositionSetIterator pawnPushes;
        inline Move operator *(void) const
        {
            Index pp = *pawnPushes;
            return Move{ int8_t(pp + (side == BLACK_SIDE ? 8 : -8)), pp };
        }

        inline void operator ++(void)
        {
            ++pawnPushes;
        }

        inline bool operator !=(const MoveIterator& psi) const
        {
            pawnPushes != psi.pawnPushes;
        }
    };
public:
    inline PawnPushGenerator(const ChessGame& cg) : chessGame{ cg } {}

    MoveIterator begin(void) const;
    MoveIterator end(void) const;
};


class chessy::KnightMoveGenerator :
    public PositionSet
{
    static const std::array<Bitboard, 64> moveSets;

public:
    inline KnightMoveGenerator(Index knight, Bitboard occupied) :
        PositionSet{ moveSets[knight] & ~occupied } {}

private:
    static Bitboard generateFromIndex(Index i);
    static std::array<Bitboard, 64> generateKnightMoves(void);
};


class chessy::


#endif // CHESSY_MOVEGENERATION_H
